@@ -16,16 +16,15 @@
 
 import re
 
+from twisted.internet import defer
+from twisted.python import log
+
 from buildbot import config
 from buildbot import interfaces
-from buildbot.interfaces import BuildSlaveTooOldError
+from buildbot.interfaces import WorkerTooOldError
 from buildbot.process import buildstep
 from buildbot.process.properties import Interpolate
 from buildbot.steps.source import Source
-from twisted.internet import defer
-from twisted.python import log
-from types import StringType
-
 
 # Notes:
 #  see
@@ -46,7 +45,7 @@ class P4(Source):
 
     name = 'p4'
 
-    renderables = ['p4base', 'p4client', 'p4viewspec', 'p4branch']
+    renderables = ['mode', 'p4base', 'p4client', 'p4viewspec', 'p4branch']
     possible_modes = ('incremental', 'full')
 
     def __init__(self, mode='incremental',
@@ -54,10 +53,13 @@ class P4(Source):
                  p4port=None, p4user=None,
                  p4passwd=None, p4extra_views=(), p4line_end='local',
                  p4viewspec=None, p4viewspec_suffix='...',
-                 p4client=Interpolate('buildbot_%(prop:slavename)s_%(prop:buildername)s'),
+                 p4client=Interpolate(
+                     'buildbot_%(prop:workername)s_%(prop:buildername)s'),
                  p4client_spec_options='allwrite rmdir',
                  p4extra_args=None,
                  p4bin='p4',
+                 use_tickets=False,
+                 stream=False,
                  **kwargs):
         self.method = method
         self.mode = mode
@@ -74,29 +76,48 @@ class P4(Source):
         self.p4client = p4client
         self.p4client_spec_options = p4client_spec_options
         self.p4extra_args = p4extra_args
+        self.use_tickets = use_tickets
+        self.stream = stream
 
-        Source.__init__(self, **kwargs)
+        super().__init__(**kwargs)
 
-        if self.mode not in self.possible_modes:
-            config.error("mode %s is not one of %s" % (self.mode, self.possible_modes))
+        if self.mode not in self.possible_modes and \
+                not interfaces.IRenderable.providedBy(self.mode):
+            config.error("mode {} is not an IRenderable, or one of {}".format(self.mode,
+                                                                              self.possible_modes))
 
         if not p4viewspec and p4base is None:
             config.error("You must provide p4base or p4viewspec")
 
         if p4viewspec and (p4base or p4branch or p4extra_views):
-            config.error("Either provide p4viewspec or p4base and p4branch (and optionally p4extra_views")
+            config.error(
+                "Either provide p4viewspec or p4base and p4branch (and optionally p4extra_views")
 
-        if p4viewspec and isinstance(p4viewspec, StringType):
-            config.error("p4viewspec must not be a string, and should be a sequence of 2 element sequences")
+        if p4viewspec and isinstance(p4viewspec, str):
+            config.error(
+                "p4viewspec must not be a string, and should be a sequence of 2 element sequences")
+
+        if not interfaces.IRenderable.providedBy(p4base) and p4base and not p4base.startswith('/'):
+            config.error('p4base should start with // [p4base = {}]'.format(p4base))
 
         if not interfaces.IRenderable.providedBy(p4base) and p4base and p4base.endswith('/'):
-            config.error('p4base should not end with a trailing / [p4base = %s]' % p4base)
+            config.error('p4base should not end with a trailing / [p4base = {}]'.format(p4base))
 
         if not interfaces.IRenderable.providedBy(p4branch) and p4branch and p4branch.endswith('/'):
-            config.error('p4branch should not end with a trailing / [p4branch = %s]' % p4branch)
+            config.error('p4branch should not end with a trailing / [p4branch = {}]'.format(
+                    p4branch))
 
         if (p4branch or p4extra_views) and not p4base:
-            config.error('If you specify either p4branch or p4extra_views you must also specify p4base')
+            config.error(
+                'If you specify either p4branch or p4extra_views you must also specify p4base')
+
+        if stream:
+            if (p4extra_views or p4viewspec):
+                config.error('You can\'t use p4extra_views not p4viewspec with stream')
+            if not p4base or not p4branch:
+                config.error('You must specify both p4base and p4branch when using stream')
+            if " " in p4base or " " in p4branch:
+                config.error('p4base and p4branch must not contain any whitespace')
 
         if self.p4client_spec_options is None:
             self.p4client_spec_options = ''
@@ -111,16 +132,25 @@ class P4(Source):
 
         d = self.checkP4()
 
+        @d.addCallback
         def checkInstall(p4Installed):
             if not p4Installed:
-                raise BuildSlaveTooOldError("p4 is not installed on slave")
+                raise WorkerTooOldError("p4 is not installed on worker")
             return 0
-        d.addCallback(checkInstall)
 
-        if self.mode == 'full':
-            d.addCallback(self.full)
-        elif self.mode == 'incremental':
-            d.addCallback(self.incremental)
+        # Try to obfuscate the password when used as an argument to commands.
+        if self.p4passwd is not None:
+            if not self.workerVersionIsOlderThan('shell', '2.16'):
+                self.p4passwd_arg = ('obfuscated', self.p4passwd, 'XXXXXX')
+            else:
+                self.p4passwd_arg = self.p4passwd
+                log.msg("Worker does not understand obfuscation; "
+                        "p4 password will be logged")
+
+        if self.use_tickets and self.p4passwd:
+            d.addCallback(self._acquireTicket)
+
+        d.addCallback(self._getAttrGroupMember('mode', self.mode))
 
         d.addCallback(self.parseGotRevision)
         d.addCallback(self.finish)
@@ -128,7 +158,7 @@ class P4(Source):
         return d
 
     @defer.inlineCallbacks
-    def full(self, _):
+    def mode_full(self, _):
         if debug_logging:
             log.msg("P4:full()..")
 
@@ -144,18 +174,21 @@ class P4(Source):
         # Then we need to sync the client
         if self.revision:
             if debug_logging:
-                log.msg("P4: full() sync command based on :base:%s changeset:%d", self._getP4BaseForLog(), int(self.revision))
-            yield self._dovccmd(['sync', '%s...@%d' % (self._getP4BaseForCommand(), int(self.revision))], collectStdout=True)
+                log.msg("P4: full() sync command based on :base:%s changeset:%d",
+                        self._getP4BaseForLog(), int(self.revision))
+            yield self._dovccmd(['sync', '{}...@{}'.format(self._getP4BaseForCommand(),
+                                                           int(self.revision))], collectStdout=True)
         else:
             if debug_logging:
-                log.msg("P4: full() sync command based on :base:%s no revision", self._getP4BaseForLog())
+                log.msg("P4: full() sync command based on :base:%s no revision",
+                        self._getP4BaseForLog())
             yield self._dovccmd(['sync'], collectStdout=True)
 
         if debug_logging:
             log.msg("P4: full() sync done.")
 
     @defer.inlineCallbacks
-    def incremental(self, _):
+    def mode_incremental(self, _):
         if debug_logging:
             log.msg("P4:incremental()")
 
@@ -166,19 +199,20 @@ class P4(Source):
         command = ['sync', ]
 
         if self.revision:
-            command.extend(['%s...@%d' % (self._getP4BaseForCommand(), int(self.revision))])
+            command.extend(['{}...@{}'.format(self._getP4BaseForCommand(), int(self.revision))])
 
         if debug_logging:
-            log.msg("P4:incremental() command:%s revision:%s", command, self.revision)
+            log.msg(
+                "P4:incremental() command:%s revision:%s", command, self.revision)
         yield self._dovccmd(command)
 
     def finish(self, res):
         d = defer.succeed(res)
 
+        @d.addCallback
         def _gotResults(results):
             self.setStatus(self.cmd, results)
             return results
-        d.addCallback(_gotResults)
         d.addCallback(self.finished)
         return d
 
@@ -197,25 +231,24 @@ class P4(Source):
             command.extend(['-p', self.p4port])
         if self.p4user:
             command.extend(['-u', self.p4user])
-        if self.p4passwd:
-            # Need to find out if there's a way to obfuscate this
-            command.extend(['-P', self.p4passwd])
+        if not self.use_tickets and self.p4passwd:
+            command.extend(['-P', self.p4passwd_arg])
         if self.p4client:
             command.extend(['-c', self.p4client])
+
         # Only add the extra arguments for the `sync` command.
         if doCommand[0] == 'sync' and self.p4extra_args:
             command.extend(self.p4extra_args)
 
         command.extend(doCommand)
-
-        command = [c.encode('utf-8') for c in command]
         return command
 
     def _dovccmd(self, command, collectStdout=False, initialStdin=None):
         command = self._buildVCCommand(command)
 
         if debug_logging:
-            log.msg("P4:_dovccmd():workdir->%s" % self.workdir)
+            log.msg("P4:_dovccmd():workdir->{}".format(self.workdir))
+
         cmd = buildstep.RemoteShellCommand(self.workdir, command,
                                            env=self.env,
                                            logEnviron=self.logEnviron,
@@ -224,20 +257,19 @@ class P4(Source):
                                            initialStdin=initialStdin,)
         cmd.useLog(self.stdio_log, False)
         if debug_logging:
-            log.msg("Starting p4 command : p4 %s" % (" ".join(command),))
+            log.msg("Starting p4 command : p4 {}".format(" ".join(command)))
 
         d = self.runCommand(cmd)
 
-        def evaluateCommand(cmd):
+        @d.addCallback
+        def evaluateCommand(_):
             if cmd.rc != 0:
                 if debug_logging:
-                    log.msg("P4:_dovccmd():Source step failed while running command %s" % cmd)
+                    log.msg("P4:_dovccmd():Source step failed while running command {}".format(cmd))
                 raise buildstep.BuildStepFailed()
             if collectStdout:
                 return cmd.stdout
-            else:
-                return cmd.rc
-        d.addCallback(lambda _: evaluateCommand(cmd))
+            return cmd.rc
         return d
 
     def _getMethod(self):
@@ -247,6 +279,7 @@ class P4(Source):
             return None
         elif self.method is None and self.mode == 'full':
             return 'fresh'
+        return None
 
     def _sourcedirIsUpdatable(self):
         # In general you should always be able to write to the directory
@@ -261,57 +294,85 @@ class P4(Source):
         builddir = self.getProperty('builddir')
 
         if debug_logging:
-            log.msg("P4:_createClientSpec() builddir:%s" % builddir)
-            log.msg("P4:_createClientSpec() SELF.workdir:%s" % self.workdir)
+            log.msg("P4:_createClientSpec() builddir:{}".format(builddir))
+            log.msg("P4:_createClientSpec() SELF.workdir:{}".format(self.workdir))
 
         prop_dict = self.getProperties().asDict()
         prop_dict['p4client'] = self.p4client
 
         client_spec = ''
-        client_spec += "Client: %s\n\n" % self.p4client
-        client_spec += "Owner: %s\n\n" % self.p4user
-        client_spec += "Description:\n\tCreated by %s\n\n" % self.p4user
-        client_spec += "Root:\t%s\n\n" % self.build.path_module.normpath(
-            self.build.path_module.join(builddir, self.workdir)
-        )
-        client_spec += "Options:\t%s\n\n" % self.p4client_spec_options
+        client_spec += "Client: {}\n\n".format(self.p4client)
+        client_spec += "Owner: {}\n\n".format(self.p4user)
+        client_spec += "Description:\n\tCreated by {}\n\n".format(self.p4user)
+        client_spec += "Root:\t{}\n\n".format(self.build.path_module.normpath(
+            self.build.path_module.join(builddir, self.workdir)))
+        client_spec += "Options:\t{}\n\n".format(self.p4client_spec_options)
         if self.p4line_end:
-            client_spec += "LineEnd:\t%s\n\n" % self.p4line_end
+            client_spec += "LineEnd:\t{}\n\n".format(self.p4line_end)
         else:
             client_spec += "LineEnd:\tlocal\n\n"
 
-        # Setup a view
-        client_spec += "View:\n"
-
-        if self.p4viewspec:
-            # uses only p4viewspec array of tuples to build view
-            # If the user specifies a viewspec via an array of tuples then
-            # Ignore any specified p4base,p4branch, and/or p4extra_views
-            suffix = self.p4viewspec_suffix or ''
-            for k, v in self.p4viewspec:
-                if debug_logging:
-                    log.msg('P4:_createClientSpec():key:%s value:%s' % (k, v))
-                client_spec += '\t%s%s //%s/%s%s\n' % (k, suffix, self.p4client, v, suffix)
+        # Perforce generates the view for stream-associated workspaces
+        if self.stream:
+            client_spec += "Stream:\t{}/{}\n".format(self.p4base, self.p4branch)
         else:
-            # Uses p4base, p4branch, p4extra_views
-            client_spec += "\t%s" % (self.p4base)
+            # Setup a view
+            client_spec += "View:\n"
 
-            if self.p4branch:
-                client_spec += "/%s" % (self.p4branch)
+            def has_whitespace(*args):
+                return any([re.search(r'\s', i) for i in args if i is not None])
 
-            client_spec += "/... //%s/...\n" % (self.p4client)
+            if self.p4viewspec:
+                # uses only p4viewspec array of tuples to build view
+                # If the user specifies a viewspec via an array of tuples then
+                # Ignore any specified p4base,p4branch, and/or p4extra_views
+                suffix = self.p4viewspec_suffix or ''
+                for k, v in self.p4viewspec:
+                    if debug_logging:
+                        log.msg('P4:_createClientSpec():key:{} value:{}'.format(k, v))
 
-            if self.p4extra_views:
-                for k, v in self.p4extra_views:
-                    client_spec += "\t%s/... //%s/%s/...\n" % (k, self.p4client, v)
+                    qa = '"' if has_whitespace(k, suffix) else ''
+                    qb = '"' if has_whitespace(self.p4client, v, suffix) else ''
+                    client_spec += '\t{}{}{}{} {}//{}/{}{}{}\n'.format(qa, k, suffix, qa, qb,
+                                                                       self.p4client, v, suffix, qb)
+            else:
+                # Uses p4base, p4branch, p4extra_views
+
+                qa = '"' if has_whitespace(self.p4base, self.p4branch) else ''
+
+                client_spec += "\t{}{}".format(qa, self.p4base)
+
+                if self.p4branch:
+                    client_spec += "/{}".format(self.p4branch)
+
+                client_spec += "/...{} ".format(qa)
+
+                qb = '"' if has_whitespace(self.p4client) else ''
+                client_spec += "{}//{}/...{}\n".format(qb, self.p4client, qb)
+
+                if self.p4extra_views:
+                    for k, v in self.p4extra_views:
+                        qa = '"' if has_whitespace(k) else ''
+                        qb = '"' if has_whitespace(k, self.p4client, v) else ''
+
+                        client_spec += "\t{}{}/...{} {}//{}/{}/...{}\n".format(qa, k, qa, qb,
+                                                                               self.p4client, v, qb)
 
-        client_spec = client_spec.encode('utf-8')  # resolve unicode issues
         if debug_logging:
             log.msg(client_spec)
 
         stdout = yield self._dovccmd(['client', '-i'], collectStdout=True, initialStdin=client_spec)
         mo = re.search(r'Client (\S+) (.+)$', stdout, re.M)
-        defer.returnValue(mo and (mo.group(2) == 'saved.' or mo.group(2) == 'not changed.'))
+        return mo and (mo.group(2) == 'saved.' or mo.group(2) == 'not changed.')
+
+    @defer.inlineCallbacks
+    def _acquireTicket(self, _):
+        if debug_logging:
+            log.msg("P4:acquireTicket()")
+
+        # TODO: check first if the ticket is still valid?
+        initialStdin = self.p4passwd + "\n"
+        yield self._dovccmd(['login'], initialStdin=initialStdin)
 
     def parseGotRevision(self, _):
         command = self._buildVCCommand(['changes', '-m1', '#have'])
@@ -324,24 +385,25 @@ class P4(Source):
         cmd.useLog(self.stdio_log, False)
         d = self.runCommand(cmd)
 
+        @d.addCallback
         def _setrev(_):
             stdout = cmd.stdout.strip()
             # Example output from p4 changes -m1 #have
-            #     Change 212798 on 2012/04/13 by user@user-unix-bldng2 'change to pickup build'
+            # Change 212798 on 2012/04/13 by user@user-unix-bldng2 'change to
+            # pickup build'
             revision = stdout.split()[1]
             try:
                 int(revision)
             except ValueError:
-                msg = ("p4.parseGotRevision unable to parse output "
-                       "of 'p4 changes -m1 \"#have\"': '%s'" % stdout)
+                msg = (("p4.parseGotRevision unable to parse output "
+                        "of 'p4 changes -m1 \"#have\"': '{}'").format(stdout))
                 log.msg(msg)
                 raise buildstep.BuildStepFailed()
 
             if debug_logging:
-                log.msg("Got p4 revision %s" % (revision,))
+                log.msg("Got p4 revision {}".format(revision))
             self.updateSourceProperty('got_revision', revision)
             return 0
-        d.addCallback(lambda _: _setrev(cmd.rc))
         return d
 
     def purge(self, ignore_ignores):
@@ -363,11 +425,11 @@ class P4(Source):
         cmd.useLog(self.stdio_log, False)
         d = self.runCommand(cmd)
 
-        def evaluate(cmd):
+        @d.addCallback
+        def evaluate(_):
             if cmd.rc != 0:
                 return False
             return True
-        d.addCallback(lambda _: evaluate(cmd))
         return d
 
     def computeSourceRevision(self, changes):
